/**
 * Copyright (c) Nicolas Gallagher.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @flow
 */

import type { LayoutEvent } from '../../__types__';

import React from 'react';
import supportsDOM from '#internal/supportsDOM';

export type LayoutConfig = {|
  box?: 'border-box' | 'content-box',
  onLayout: ?(e: LayoutEvent) => void
|};

const HANDLE_RESIZE_KEY = '__reactHandleResize';
let didWarn = false;
let resizeObserver = null;

function getResizeObserver(): ?ResizeObserver {
  if (supportsDOM() && typeof window.ResizeObserver !== 'undefined') {
    if (resizeObserver == null) {
      resizeObserver = new window.ResizeObserver(function (entries) {
        entries.forEach((entry) => {
          const handleResize = entry.target[HANDLE_RESIZE_KEY];
          if (handleResize != null) {
            handleResize(entry);
          }
        });
      });
    }
  } else if (
    process.env.NODE_ENV !== 'production' &&
    process.env.NODE_ENV !== 'test'
  ) {
    if (!didWarn) {
      console.warn(
        'onLayout relies on ResizeObserver which is not supported by your browser. ' +
          'Please include a polyfill, e.g., https://github.com/que-etc/resize-observer-polyfill.'
      );
      didWarn = true;
    }
  }
  return resizeObserver;
}

// TODO(#37): Support multiple uses
export function useLayout(config: LayoutConfig) {
  const observer = getResizeObserver();
  const { box = 'border-box', onLayout } = config;

  const attachedNode = React.useRef(null);

  return React.useCallback(
    (target: ?any) => {
      if (target != null) {
        attachedNode.current = target;
        if (onLayout != null) {
          target[HANDLE_RESIZE_KEY] = function (entry) {
            const { borderBoxSize, contentRect } = entry;
            // 'contentRect' is legacy but not deprecated, and it provides the same
            // values as 'contentBoxSize' in fewer btyes
            let { height, width } = contentRect;
            if (box === 'border-box') {
              if (borderBoxSize != null) {
                // Firefox doesn't provide a sequence of boxes, even though the spec
                // defines this value as a frozen array
                const borderSize = borderBoxSize[0] || borderBoxSize;
                height = borderSize.blockSize;
                width = borderSize.inlineSize;
              } else {
                // 'borderBoxSize' is not currently available in all engines and not
                // included in most polyfills, so we fallback to 'getBoundingClientRect'
                const borderRect = target.getBoundingClientRect();
                height = borderRect.height;
                width = borderRect.width;
              }
            }
            onLayout({ width, height });
          };
        }
        if (observer != null) {
          if (typeof target[HANDLE_RESIZE_KEY] === 'function') {
            // $FlowExpectedError: the Flow type is wrong
            observer.observe(target);
          }
        }
      } else if (attachedNode.current && observer != null) {
        observer.unobserve(attachedNode.current);
        attachedNode.current = null;
      }
    },
    [box, observer, onLayout]
  );
}
